/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/


////////////////////////////////////////////////////////////////////////////
//
// Assembly Methods
//
////////////////////////////////////////////////////////////////////////////

#include <libc.h>
#include <stdio.h>
#include <strings.h>
#include <List.h>

#include "oo7.h"
#include "globals.h"
#include "GenParams.h"
#include "VarParams.h"
#include "baidlist.h"

extern int	nextComplexAssemblyId;
extern int	nextBaseAssemblyId;
extern int	debugMode;
extern char*	types[NumTypes];

extern BAIdList* private_cp;
extern BAIdList* shared_cp;

////////////////////////////////////////////////////////////////////////////
//
// Assembly Constructor  (required by Ontos)
//
////////////////////////////////////////////////////////////////////////////

Assembly::Assembly(APL *theAPL) : Object(theAPL) {
    initDirectType(ASSType);
}


////////////////////////////////////////////////////////////////////////////
//
// getDirectType required by ONTOS.
//
////////////////////////////////////////////////////////////////////////////

Type* Assembly::getDirectType() {
    return ASSType;
}


////////////////////////////////////////////////////////////////////////////
//
// ComplexAssembly Constructor
//
////////////////////////////////////////////////////////////////////////////

ComplexAssembly::ComplexAssembly(int 		  asId,
				 Module*	  module,
    	                         ComplexAssembly* parentAssembly,
		                 int 		  levelNo)
{
    int 		i, nextId;
    ComplexAssembly*	subAssembly;   
    BaseAssembly*	baseAssembly;

    if (debugMode) {
        printf("ComplexAssembly::ComplexAssembly(asId = %d, levelNo = %d)\n",
                asId, levelNo);
    }

    // initialize the direct type (ONTOS requirement.)
    initDirectType(CAType);

    // initialize the simple stuff
    id = asId;
    int typeNo = (int) (random() % NumTypes);
    strncpy(type, types[typeNo], TypeSize);
    buildDate = MinAssmDate + (int) (random() % (MaxAssmDate-MinAssmDate+1));

    // set assemblyReference to reference parent.
    assemblyReference.Reset(parentAssembly);

    // create a new list and set assocReference to it.
    subAssembliesReference.Reset(
        new (oo7Group, OC_list) List(ASSType));

    assmType = Complex;

    // recursively create subassemblies for this complex assembly
    for (i = 0; i < NumAssmPerAssm; i++) 
    {
        if (levelNo < NumAssmLevels-1) 
	{
	    // create a complex assembly as the subassembly
            nextId = nextComplexAssemblyId++;
	    subAssembly = new (oo7Group, CAType)
		ComplexAssembly(nextId, module, this, levelNo+1);
	    subAssemblies()->Insert(subAssembly);
            // write the new assembly to DB.  (Ontos specific.)
            subAssembly->putObject();
        } 
	else 
	{
	    // create a base assembly as the subassembly
	    // and insert its id into a hash index
            nextId = nextBaseAssemblyId++;
	    baseAssembly = new (oo7Group, BAType)
		BaseAssembly (nextId, module, this);
	    subAssemblies()->Insert(baseAssembly);
            // write the assembly to disk.  (Ontos specific.)
            baseAssembly->putObject();
            // add to set of all base assemblies.
	    AllBaseAssemblies->Insert(nextId, baseAssembly); 
        }
    }
    // write the subAssemblies association to disk.  (ONTOS.)
    subAssemblies()->putObject();
}

// Additional constructor required by Ontos.
ComplexAssembly::ComplexAssembly(APL *theAPL) : Assembly(theAPL) {}

ComplexAssembly::~ComplexAssembly()
{
    Destroy (FALSE);
}

void ComplexAssembly::Destroy(OC_Boolean aborted)
{
    if (aborted)
      Assembly::Destroy (aborted);
}

void ComplexAssembly::deleteObject(OC_Boolean deallocate)
{
    Assembly::deleteObject (deallocate);
}

////////////////////////////////////////////////////////////////////////////
//
// getDirectType required by ONTOS.
//
////////////////////////////////////////////////////////////////////////////

Type* ComplexAssembly::getDirectType() {
    return CAType;
}



////////////////////////////////////////////////////////////////////////////
//
// BaseAssembly Constructor
//
////////////////////////////////////////////////////////////////////////////

BaseAssembly::BaseAssembly(int     	asId,
			   Module* 	module,
			   ComplexAssembly* parentAssembly)
{
    int 		compId;

    if (debugMode) {
        printf("BaseAssembly::BaseAssembly(asId = %d, levelNo = %d)\n",
                asId, NumAssmLevels);
    }
    // initialize the direct type (ONTOS requirement.)
    initDirectType(BAType);

    // initialize the simple stuff
    id = asId;
    int typeNo = (int) (random() % NumTypes);
    strncpy(type, types[typeNo], TypeSize);
    buildDate = MinAssmDate + (int) (random() % (MaxAssmDate-MinAssmDate+1));

    assemblyReference.Reset(parentAssembly);

    privReference.Reset(
        new (oo7Group, OC_list) List(CPType), this);

    sharReference.Reset(
        new (oo7Group, OC_list) List(CPType), this);

    assmType = Base;
    module->allBases()->Insert(this); // add the base assembly to the set
		                      // of all base assemblies in this module
    module->allBases()->putObject();  // write set to disk.  (ONTOS.)

    // get access to the design library containing composite parts
    int i;
    int lowCompId = (module->id - 1) * NumCompPerModule + 1;
    int compIdLimit = NumCompPerModule;

    // first select the private composite parts for this assembly
    if (debugMode) { printf("[Private parts = { "); }
    for (i = 0; i < NumCompPerAssm; i++) 
    {
	compId = lowCompId + (int) (random() % compIdLimit);
	private_cp[compId].insert(id); // keep track of which CP
           // uses this base assembly as private
        if (debugMode) { printf("%d ", compId); }
    }

    // next select the shared composite parts for this assembly
    if (debugMode) { printf("}, Shared parts = { "); }
    for (i = 0; i < NumCompPerAssm; i++) 
    {
	compId = (int) (random() % TotalCompParts) + 1;
	shared_cp[compId].insert(id); // keep track of which CP
               // uses this base assembly as shared
        if (debugMode) { printf("%d ", compId); }
    }
    if (debugMode) { printf("}\n"); }
}

// Additional constructor required by Ontos.
BaseAssembly::BaseAssembly(APL *theAPL) : Assembly(theAPL) {}

BaseAssembly::~BaseAssembly()
{
    Destroy (FALSE);
}

void BaseAssembly::Destroy(OC_Boolean aborted)
{
    if (aborted)
      Assembly::Destroy (aborted);
}

void BaseAssembly::deleteObject(OC_Boolean deallocate)
{
    Assembly::deleteObject (deallocate);
}


////////////////////////////////////////////////////////////////////////////
//
// getDirectType required by ONTOS.
//
////////////////////////////////////////////////////////////////////////////

Type* BaseAssembly::getDirectType() {
    return BAType;
}


//////////////////////////////////////////////////////////////////////////
// 
// DoNothing method for Assemblies 
// 
//////////////////////////////////////////////////////////////////////////

void Assembly::DoNothing ()
{
    if (id < 0) 
        printf("DoNothing - negative id\n");

    if (debugMode) {
        printf("==> DoNothing(id = %d, type = %s, buildDate = %d)\n",
                               id, type, buildDate);
    }
}


/*  $Version: Assembly.C 1.4 93/02/25 10:08:35 Exp $ */

