/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <malloc.h>
#include <stdio.h>
#include <strings.h>
#include "oo7.h"
#include "globals.h"
#include "GenParams.h"
#include "VarParams.h"

////////////////////////////////////////////////////////////////////////////
//
// Document Methods
//
////////////////////////////////////////////////////////////////////////////

extern int	debugMode;

////////////////////////////////////////////////////////////////////////////
//
// Document Constructor
//
////////////////////////////////////////////////////////////////////////////

Document::Document(int cpId, CompositePart* cp)
{
    char	myText[DocTextLength];
    char 	docTitle[TitleSize];
    int         curChar, strChar;
    int         stringLen;

 
    // initialize direct type --- required by Ontos.
    initDirectType(DocumentType);

    // fill in id
    id = cpId; 

    // fill in back pointer to composite part
    compPartReference.Reset(cp); 

    // prepare and fill in the document title
    sprintf(docTitle, "Composite Part %08d", cpId);
    if (debugMode) {
        printf("Document::Document(title = %s)\n", docTitle);
    }
    strncpy(title, docTitle, TitleSize);

    text = new char[DocumentSize];

    sprintf(myText, DocumentText, cpId);
    stringLen = strlen(DocumentText);

    curChar = strChar = 0;
    while (curChar < DocumentSize-1) {
        text[curChar++] = myText[strChar++];
        if (strChar >= stringLen) { strChar = 0; }
    }
    text[curChar] = '\0';

    textLen = strlen(text);
}

// Additional constructor required by Ontos.
Document::Document(APL *theAPL) : Object(theAPL) {}

// required by Ontos
Type* Document::getDirectType() {
//    return (Type*) OC_lookup("Document");
    return DOCType;
}


// destructor
Document::~Document()
{
// all index maintenance is now done by CompositePart methods.

    // remove document from extent of all documents
//    AllDocuments.remove (this);

    // put the set
//    AllDocuments.putObject();
}

////////////////////////////////////////////////////////////////////////////
//
// Document searchText Method for use in traversals
//
////////////////////////////////////////////////////////////////////////////

int Document::searchText(char c)
{
    char x;
	
    if (debugMode) {
        printf("                    Document::searchText(title = %s)\n", title);
    }
    x = c;

    // count occurrences of the indicated letter (for busy work)
    int i = 0; 
    int count = 0;
    while (text[i] != '\0') {
	if (text[i++] == x) { count++; }
    }

    if (debugMode) {
        printf("                    [found %d '%c's among %d characters]\n",
	        		     count, c, i);
    }
    return count;
}


////////////////////////////////////////////////////////////////////////////
//
// Document replaceText Method for use in traversals
//
////////////////////////////////////////////////////////////////////////////

int Document::replaceText(char *oldString, char *newString)
{
    char *newText;

    if (debugMode) {
        printf("                    Document::changeText(title = %s)\n", title);
    }

    // check to see if the text starts with the old string

    int oldTextLength = strlen(text) + 1;
    int oldStrLength  = strlen(oldString);
    int foundMatch = (strncmp(text, oldString, oldStrLength) == 0);

    // if so, change it to start with the new string instead
    if (foundMatch) 
    {
        int newStrLength  = strlen(newString);
        int lengthDiff = newStrLength - oldStrLength;
        int newTextLength  = oldTextLength + lengthDiff;

	if (lengthDiff == 0) {
            strncpy(text, newString, newStrLength);
        } else {
            newText = (char *) malloc(newTextLength);
            strncpy(newText, newString, newStrLength);
            strncpy(&newText[newStrLength], &text[oldStrLength],
			oldTextLength - oldStrLength);
	    delete text;
            text = strdup (newText);
/*
was
	    text = newText;
*/
	}
        // put the change to the DB
        this->putObject();
    }

    if (debugMode) 
    {
	if (foundMatch) {
            printf("                    [changed \"%s\" to \"%s\"]\n",
	                                 oldString, newString);
	} else {
            printf("                    [no match, so no change was made]\n");
	}
    }

    if (foundMatch) {
        return 1;

    } else {
        return 0;
    }
}



/*  $Version: Document.C 1.2 93/02/22 15:59:37 Exp $ */

