/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <stdio.h>
#include <List.h>

#include "oo7.h"
#include "globals.h"
#include "GenParams.h"
#include "BenchParams.h"
#include "VarParams.h"

//////////////////////////////////////////////////////////////////
//
// Insert - Create NumNewCompParts new composite parts (which
// means also creating a set of AtomicParts for each.)  Add
// a reference to each new composite part from a randomly chosen
// base assembly (a different random choice of base assembly for
// each new composite part.)
//
//////////////////////////////////////////////////////////////////

extern int	debugMode;
extern int	nextAtomicId;
extern int	nextCompositeId;
extern int	nextBaseAssemblyId;

//////////////////////////////////////////////////////////////////
//
// Routine to do Insert
//
//////////////////////////////////////////////////////////////////

void insert1()
{
    // now create the desired number of new composite parts, adding each
    // one as a (private) composite part that a randomly selected base
    // assembly now wishes to use

    int 		compId;
    int 		assmId;
    CompositePart*	compH;
    BaseAssembly*  	assmH;

    if (debugMode) {
	printf("In Insert, nextCompositeId = %d:\n", nextCompositeId);
	printf("In Insert, nextAtomicId = %d:\n", nextAtomicId);
	printf("In Insert, nextBaseAssemblyId = %d:\n", nextBaseAssemblyId);
    }

    for (int i = 0; i < NumNewCompParts; i++) {

	// add a new composite part to the design library
        if (debugMode) {
	    printf("In Insert, making composite part %d:\n", nextCompositeId);
	}

        compId = nextCompositeId++;
//	compH = new (oo7Group, (Type*) OC_lookup ("CompositePart"))
//		CompositePart(compId);
	compH = new (oo7Group, CPType)
		CompositePart(compId);

        // insert into extent of composite parts.
        AllCompPartsById->Insert(compId, compH);

        // put the changes to the set to DB.
// GKK
#ifndef MARKMODIFIED
        AllCompPartsById->putObject();
#else
        AllCompPartsById->markModified();
#endif

	// randomly select a base assembly that should use it and figure
	// out which module it resides in
        int baseAssembliesPerModule = (nextBaseAssemblyId-1)/TotalModules;
	assmId = (int) (random() % (nextBaseAssemblyId-1)) + 1;

	// now locate the appropriate base assembly object within its module
	assmH = (BaseAssembly*)(Entity*) 
                 AllBaseAssemblies->getEntityElement(assmId);
	if (assmH == NULL) 
	{
            fprintf(stderr, "ERROR: Can't find base assembly %d\n", assmId);
            OC_transactionAbort();
            exit(1);
        }

        // finally, add the newly created composite part as a privately used
        // member of the randomly selected base assembly

	// first add this assembly to the list of assemblies in which
	// this composite part is used as a private member
	(compH->usedInPriv())->Insert(assmH);
        // put the association to the disk
        (compH->usedInPriv())->putObject();

	// then add the composite part compH to the list of private parts used
	// in this assembly
	assmH->componentsPriv()->Insert(compH);
        // put the updated list to the db
      	assmH->componentsPriv()->putObject();

        // put the new composite part to the disk
        compH->putObject();

        if (debugMode) {
	    printf("[just made it be used by base assembly %d]\n", assmId);
        }
    }
}


/*  $Version: Insert.C 1.3 93/02/25 10:08:43 Exp $ */

