/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <malloc.h>
#include <stdio.h>
#include <strings.h>
#include "oo7.h"
#include "globals.h"
#include "GenParams.h"
#include "VarParams.h"

////////////////////////////////////////////////////////////////////////////
//
// Manual Methods
//
////////////////////////////////////////////////////////////////////////////

extern int	debugMode;

////////////////////////////////////////////////////////////////////////////
//
// Manual Constructor
//
////////////////////////////////////////////////////////////////////////////

Manual::Manual(int modId, Module* modPtr)
{
    char	myText[ManualTextLength];
    char 	docTitle[TitleSize];
    int         curChar, strChar;
    int         stringLen;
 
    // initialize direct type --- required by Ontos.
    initDirectType(ManualType);

    // fill in id
    id = modId; 

    // fill in back pointer to composite part
    moduleReference.Reset(modPtr); 

    // prepare and fill in the document title
    sprintf(docTitle, "Manual         %08d", modId);
    if (debugMode) {
        printf("Manual::Manual(title = %s)\n", docTitle);
    }
    strncpy(title, docTitle, TitleSize);

    text = new char[ManualSize];

    sprintf(myText, ManualText, modId);
    stringLen = strlen(myText);

    curChar = strChar = 0;
    while (curChar < ManualSize-1) {
        text[curChar++] = myText[strChar++]; 
        if (strChar >= stringLen) { strChar = 0; }
    }
    text[curChar] = '\0'; 

    textLen = strlen(text);
}

// Additional constructor required by Ontos.
Manual::Manual(APL *theAPL) : Object(theAPL) {}

// required by Ontos
Type* Manual::getDirectType() {
//    return (Type*) OC_lookup("Manual");
    return MANType;
}


// destructor
Manual::~Manual()
{
}


////////////////////////////////////////////////////////////////////////////
//
// Manual searchText Method for use in traversals
//
////////////////////////////////////////////////////////////////////////////

int Manual::searchText(char c)
{
    char x;
	
    if (debugMode) {
        printf("                    Manual::searchText(title = %s)\n", title);
    }
    x = c;

    // count occurrences of the indicated letter (for busy work)
    int i = 0; 
    int count = 0;
    while (text[i] != '\0') {
	if (text[i++] == x) { count++; }
    }

    if (debugMode) {
        printf("                    [found %d '%c's among %d characters]\n",
	        		     count, c, i);
    }
    return count;
}


////////////////////////////////////////////////////////////////////////////
//
// Manual replaceText Method for use in traversals
//
////////////////////////////////////////////////////////////////////////////

int Manual::replaceText(char *oldString, char *newString)
{
    char *newText;

    if (debugMode) {
        printf("                    Manual::changeText(title = %s)\n", title);
    }

    // check to see if the text starts with the old string

    int oldTextLength = strlen(text) + 1;
    int oldStrLength  = strlen(oldString);
    int foundMatch = (strncmp(text, oldString, oldStrLength) == 0);

    // if so, change it to start with the new string instead
    if (foundMatch) 
    {
        int newStrLength  = strlen(newString);
        int lengthDiff = newStrLength - oldStrLength;
        int newTextLength  = oldTextLength + lengthDiff;

	if (lengthDiff == 0) {
            strncpy(text, newString, newStrLength);
        } else {
            newText = (char *) malloc(newTextLength);
            strncpy(newText, newString, newStrLength);
            strncpy(&newText[newStrLength], &text[oldStrLength],
			oldTextLength - oldStrLength);
	    delete text;
            text = strdup (newText);
/*
was
	    text = newText;
*/
	}
        // put the change to the DB
        this->putObject();
    }

    if (debugMode) 
    {
	if (foundMatch) {
            printf("                    [changed \"%s\" to \"%s\"]\n",
	                                 oldString, newString);
	} else {
            printf("                    [no match, so no change was made]\n");
	}
    }

    if (foundMatch) {
        return 1;

    } else {
        return 0;
    }
}

////////////////////////////////////////////////////////////////////////////
//
// Manual firstLast Method for use in traversals
//
////////////////////////////////////////////////////////////////////////////

int Manual::firstLast()
{

    return(text[textLen-1] == text[0]);
}

/*  $Version: Manual.C 1.3 93/02/22 15:59:46 Exp $ */

