/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <stdio.h>

#include "oo7.h"
#include "globals.h"
#include "GenParams.h"
#include "BenchParams.h"

//////////////////////////////////////////////////////////////////
//
// Query #2 - 1% selection on AtomicParts via build date (the
// most recent 1% of AtomicParts.)  A B+ tree index is used.
//
// This is the hand-coded (no SQL) version.
//
/////////////////////////////////////////////////////////////////


extern int	debugMode;


//////////////////////////////////////////////////////////////////
//
// Routine to do Query #2
//
//////////////////////////////////////////////////////////////////


int query2()
{
    DictionaryIterator* partItr; // iterator for traversing AllParts B-tree
    AtomicPart* ap;

    // choose a range of dates with the appropriate selectivity
    int dateRange = (Query2Percent * (MaxAtomicDate - MinAtomicDate)) / 100;
    int lowerDate = MaxAtomicDate - dateRange;

    if (debugMode) {
        printf("    Running Query2 with dateRange = %d, lowerDate = %d\n",
		    dateRange, lowerDate);
    }

    // get the dictionary iterator.  (ONTOS class.)
    partItr = new DictionaryIterator(
                    AllPartsByDate,       // use this B-tree
                    FALSE,                // get elements, not tags
                    lowerDate,            // start iteration here
                    MaxAtomicDate);       // end iteration here.

    int count = 0;

    while (partItr->moreData()) {
	ap = (AtomicPart *)(Entity*)(*partItr)(); 
	if (debugMode) {
	    printf("    In Query2, partId = %d, buildDate = %d\n",
			ap->id, ap->buildDate);
	}

	// process qualifying parts by calling the null procedure
	if (ap->buildDate >= lowerDate) {
	    ap->DoNothing();
	    count++;
	}
	else // shouldn't happen
	{  
	    printf("error:    In Query2, partId = %d, buildDate = %d\n",
			ap->id, ap->buildDate);
	}
    }
    return count;
}


/*  $Version: Query2.C 1.1 93/02/19 16:07:19 Exp $ */

