/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <stdio.h>
#include <List.h>

#include "oo7.h"
#include "globals.h"
#include "GenParams.h"
#include "BenchParams.h"

//////////////////////////////////////////////////////////////////
//
// Query #6 - find all assemblies (base or complex) B that "use" 
// (directly or transitively ) a composite part with a more recent 
// build date than B's build date.
//
// "hand-coded" version (no SQL.)
//
/////////////////////////////////////////////////////////////////


extern int	debugMode;


//////////////////////////////////////////////////////////////////
//
// Recursive Workhorse Routine for Query #6
//
//////////////////////////////////////////////////////////////////


int checkMaxCompDate(Assembly* assmH, int& count)
{
    ListIterator* assmI; // iterator for traversing base assemblies
    ListIterator* compI; // iterator for traversing composite parts
    Assembly* subAssmH;
    BaseAssembly* baseH;  
    ComplexAssembly* compH;
    CompositePart* cpH;
    int rsltDate;

    // check to see if this assembly uses a more recently dated composite
    // part, and process it if so - handling complex and base assemblies
    // differently since the former have subassemblies while the latter
    // directly use composite parts

    int maxSubDate = -1;

    if (assmH->assmType == Complex)
    {
        // complex assembly case - check its subassemblies recursively

        if (debugMode) {
            printf("In Query6:  complex assembly %d, buildDate = %d\n",
                    assmH->id, assmH->buildDate);
        }
	compH = (ComplexAssembly*) assmH;
        assmI = new ListIterator(compH->subAssemblies());
	while (assmI->moreData())
	{
            subAssmH = (Assembly*)(Entity*)(*assmI)();
	    rsltDate = checkMaxCompDate(subAssmH, count);
	    if (rsltDate > maxSubDate) maxSubDate = rsltDate;
	}
	delete assmI;
    } 
    else 
    {
        // base assembly case - check its composite part build dates
        if (debugMode) {
            printf("In Query6:  base assembly %d, buildDate = %d\n",
                    assmH->id, assmH->buildDate);
            printf("    composite part dates = { ");
        }
	baseH = (BaseAssembly*) assmH;
        compI = new ListIterator(baseH->componentsPriv());
	while (compI->moreData())
	{
            cpH = (CompositePart*)(Entity*)(*compI)();
            if (debugMode) { printf("%d ",cpH->buildDate); }
	    if (cpH->buildDate > maxSubDate) maxSubDate = cpH->buildDate;
        }
	delete compI;
        if (debugMode) { printf("}\n"); }
    }

    // see if the maximum build date of composite parts used by this assembly
    // is greater than the assembly's build date, and process the assembly if
    // this is found to be the case;  also, return the maximum build date for
    // this subassembly

    if (maxSubDate > assmH->buildDate) {
        assmH->DoNothing();
	count++;
    }
    return(maxSubDate);
}


//////////////////////////////////////////////////////////////////
//
// Top-Level Routine to do Query #6
//
//////////////////////////////////////////////////////////////////


int query6()
{
    DictionaryIterator* modI; // ONTOS iterator for traversing 
                              // AllModulesByName collection
    Module* modH;  

    // scan modules, visiting all of the assemblies in each one, checking
    // to see if they use (either directly or indirectly) a more recently
    // dated composite part;  the visits are done by recursively traversing
    // the assemblies, checking them on the way back up (to avoid lots of
    // repeated work)

    modI = new DictionaryIterator(AllModulesByName); // defaults to full scan
    int count = 0;
    while (modI->moreData()) {
        modH = (Module*) (Entity*) (*modI)();
	int rslt = checkMaxCompDate(modH->designRoot(), count);
    }
    delete modI;
    return count;
}

/*  $Version: Query6.C 1.1 93/02/19 16:07:29 Exp $ */

