/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <string.h>
#include <stdio.h>
#include <Group.h>

#include "oo7.h"
#include "globals.h"
#include "GenParams.h"
#include "BenchParams.h"
#include "VarParams.h"

//////////////////////////////////////////////////////////////////
//
// Query #8 --- value join between Documents and AtomicParts
// on Document.id and AtomicPart.docId.  This is a key - foreign
// key join.  Implementation is via index nested loops join
//
// The main drawback is that we have no semantics whatsoever for
// this query.
//
// This is the hand-coded (no SQL) version.
//
//////////////////////////////////////////////////////////////////


extern int	debugMode;

extern void JoinDoNothing(int x, int y);


//////////////////////////////////////////////////////////////////
//
// Routine to do Query #8
//
//////////////////////////////////////////////////////////////////


int query8()
{
#ifdef ORIG
    // ONTOS iterator for traversing AllPartsById collection
    DictionaryIterator* partItr; 
    // ONTOS iterator for traversing AllDocumentsByTitle collection
    DictionaryIterator* docItr;  
    AtomicPart* ap;
    Document* doc;
    int docId;

    int count = 0;

    // defaults to full scan
    partItr = new DictionaryIterator(AllPartsById);  
    while (partItr->moreData())
    {
        // get the atomic part
        ap = (AtomicPart*)(Entity*)(*partItr)();
	docId = ap->docId;
//	printf("looking for document with id =%d\n",docId);

	// look up the matching document, if any
	doc = (Document*)(Entity*) AllDocumentsById->getEntityElement(docId);
	if (doc != NULL)
	{
	   // found a matching pair of tuples
           if (debugMode) 
                    printf("In Query8, atomic part %d matches doc %d.\n",
			            ap->id, doc->id);
	   JoinDoNothing(ap->id, doc->id);
	   count++;
    	}
	// done with current document
   }
   delete partItr;  // clean up the all part iterator
   return count;
#else
    // ONTOS iterator for traversing AllPartsById collection
    GroupInstanceIterator* partItr; // ONTOS iterator for traversing 
    // ONTOS iterator for traversing AllDocumentsByTitle collection
    DictionaryIterator* docItr;  
    AtomicPart* ap;
    Document* doc;
    int docId;

    int count = 0;

    // defaults to full scan
    partItr = oo7Group->getInstances (APType);
    while (partItr->moreData())
    {
        // get the atomic part
        ap = (AtomicPart*)(*partItr)();
	docId = ap->docId;
//	printf("looking for document with id =%d\n",docId);

	// look up the matching document, if any
	doc = (Document*)(Entity*) AllDocumentsById->getEntityElement(docId);
	if (doc != NULL)
	{
	   // found a matching pair of tuples
           if (debugMode) 
                    printf("In Query8, atomic part %d matches doc %d.\n",
			            ap->id, doc->id);
	   JoinDoNothing(ap->id, doc->id);
	   count++;
    	}
	// done with current document
   }
   delete partItr;  // clean up the all part iterator
   return count;
#endif
}

/*  $Version: Query8.C 1.1 93/02/19 16:07:32 Exp $ */

