/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/


#include <libc.h>
#include <string.h>
#include <stdio.h>
#include <List.h>

#include "oo7.h"
#include "globals.h"
#include "GenParams.h"
#include "BenchParams.h"
#include "VarParams.h"
#include "PartIdList.h"


//////////////////////////////////////////////////////////////////
//
// Reorganization Test #2 - Randomly delete and reinsert as many
// atomic parts per composite part as each one has, interleaving
// reorganizations of all of the composite parts, to test out the
// deletion/reallocation handling and performance characteristics
// of the system
//
//////////////////////////////////////////////////////////////////


extern char*	types[NumTypes];
extern int	debugMode;


//////////////////////////////////////////////////////////////////
//
// Routine to do Reorg2
//
//////////////////////////////////////////////////////////////////


int reorg2()
{
    CompositePart* cp;

    // for as many atomic parts as each composite part contains,
    // iterate through all the composite parts, deleting and then
    // recreating one of the atomic parts from its atomic parts graph
    // (and count the number of atomic parts processed in this way)

    int count = 0;
    for (int i = 0; i < NumAtomicPerComp; i++) 
    {
        // create an Ontos "DictionaryIterator"
        DictionaryIterator *compPartItr;

        // creating it with defaults means the iterator will
        // range over all composite parts in the iterator.
        compPartItr = new DictionaryIterator(AllCompPartsById);

    	while (compPartItr->moreData())
    	{
            // crystal-clear Ontos syntax follows
            cp = (CompositePart*)(Entity*)(*compPartItr)();

            if (debugMode) {
                printf("    In Reorg2, partId = %d:\n", cp->id);
            }
            count += cp->reorg2();
        }
	delete compPartItr;
    }
    return count;
}



//////////////////////////////////////////////////////////////////
//
// Composite Part Methods for Reorg2
//
//////////////////////////////////////////////////////////////////


int CompositePart::reorg2()
{
    AtomicPart* 	victim;
    ListIterator* 	connI;
    PartIdList 		toList;
    PartIdList 		fromList;
    Connection* 	cn;

    if (debugMode) {
        printf("CompositePart::Reorg2(id = %d)\n", id);
    }

    // first select and locate a victim atomic part

    int lowId = rootPart()->id;
    int victimId = lowId + (int) (random() % NumAtomicPerComp);

    victim = (AtomicPart*) (Entity*)AllPartsById->Remove(victimId);
    if (victim == NULL) {
            fprintf(stderr, "ERROR: Unable to find atomic part %d\n", victimId);
            OC_transactionAbort();
            exit(1);
    }


    // next record the affected to and from connections

    // first do the to connections
    connI = new ListIterator(victim->to());
    while (connI->moreData())    {
        cn = (Connection*)(Entity*)(*connI)();
	toList.insert(cn->to()->id);
    }
    delete connI;

    // then do the from connections
    connI = new ListIterator(victim->from());
    while (connI->moreData())
    {
        cn = (Connection*)(Entity*)(*connI)();
	toList.insert(cn->from()->id);
    }
    delete connI;

    // now do the actual part deletion

    if (debugMode) {
        printf("\tReorg2: Deleting atomic part %d\n", victim->id);
    }

    Type* victimType = victim->getDirectType();

    victim->deleteObject();

    // time to (approximately) recreate the deleted part
    victim = new (oo7Group, victimType) AtomicPart (victimId, this);

    if (victimId == lowId) rootPartReference.Reset(victim, this);
    
    if (debugMode) {
        printf("\tReorg2: Recreated atomic part %d\n", victim->id);
    }

    // finally, connect the part back up to the other parts again

    int partId;
    AtomicPart* part;
//    Type* connectionType = (Type*) OC_lookup ("Connection");
    Type* connectionType = CONType;

    while (!toList.empty()) {
	partId = toList.remove();
	part = (AtomicPart*)(Entity*) AllPartsById->getEntityElement(partId);
	cn = new (oo7Group, connectionType) Connection (victim, part);
        // put this new connection into the db.
        cn->putObject();
        // put the updated atom into the db
        part->putObject();
    }

    while (!fromList.empty()) {
	partId = fromList.remove();
	part = (AtomicPart*) (Entity*) AllPartsById->getEntityElement(partId);
	cn = new (oo7Group, connectionType) Connection (part, victim);
        // put this new connection into the db.
        cn->putObject();
        // put the updated atom into the db
        part->putObject();
    }

    // put (newly created) victim into db
    victim->putObject();

    return 1;
}

/*  $Version: Reorg2.C 1.3 93/02/25 10:08:49 Exp $ */

